#
# spec file
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor serial%{nil}

%define pname quantum-espresso

%if 0%{?sle_version} >= 150200
%define DisOMPI1 ExclusiveArch:  do_not_build
%endif
%if !0%{?is_opensuse} && 0%{?sle_version:1} && 0%{?sle_version} < 150200
%define DisOMPI3 ExclusiveArch:  do_not_build
%endif

%if "%{flavor}" == ""
%define package_name %{pname}
ExclusiveArch:  do_not_build
%endif

%if "%{flavor}" != "" && !0%{?DisOMPI1:1} && !0%{?DisOMPI2:1} && !0%{?DisOMPI3:1}
ExclusiveArch:  x86_64
%endif

%if "%{flavor}" == "mvapich2"
%global mpi_flavor mvapich2
%endif

%if "%{flavor}" == "openmpi4"
%global mpi_flavor openmpi
%define mpi_vers 4
%{?DisOMPI4}
%endif

%if "%{flavor}" == "openmpi5"
%global mpi_flavor openmpi
%define mpi_vers 5
%{?DisOMPI5}
%endif

%{?mpi_flavor:%{bcond_without mpi}}%{!?mpi_flavor:%{bcond_with mpi}}
%{?with_mpi:%{!?mpi_flavor:error "No MPI family specified!"}}

%define mpi_ext %{?mpi_vers}

%if %{without mpi}
 %define my_prefix %_prefix
 %define my_bindir %_bindir
 %define my_libdir %_libdir
 %define my_incdir %_includedir
%else
 %define my_prefix %{_libdir}/mpi/gcc/%{mpi_flavor}%{?mpi_ext}
 %define my_bindir %{my_prefix}/bin
 %define my_libdir %{my_prefix}/%{_lib}
 %define my_suffix -%{mpi_flavor}%{?mpi_ext}
%endif

%if 0%{!?package_name:1}
 %define package_name   %pname%{?my_suffix}
%endif

%global devlibx_version 0.1.0
# Unimportant rank mismatch issues that otherwise cause builds to fail for GCC >= 10
%if 0%{?suse_version} > 1500
%global extra_gfortran_flags -fallow-argument-mismatch
%else
%global extra_gfortran_flags %{nil}
%endif
# We need to turn off "-Werror=return-type" in optflags to avoid build failures
%global optflags %(echo "%{optflags}" | sed "s/ -Werror=return-type//")
Name:           %{package_name}
Version:        6.8
Release:        3.3
Summary:        A suite for electronic-structure calculations and materials modeling
License:        GPL-2.0-only
Group:          Productivity/Scientific/Physics
URL:            http://www.quantum-espresso.org
Source0:        https://gitlab.com/QEF/q-e/-/archive/qe-%{version}/q-e-qe-%{version}.tar.bz2
Source1:        https://gitlab.com/max-centre/components/devicexlib/-/archive/%{devlibx_version}/devicexlib-%{devlibx_version}.tar.gz
Source2:        %{pname}-rpmlintrc
# PATCH-FEATURE-OPENSUSE quantum-espresso-devxlib-no-download.patch badshah400@gmail.com -- Do not try to download devxlib, use SOURCE1 instead.
Patch1:         quantum-espresso-devxlib-no-download.patch
BuildRequires:  blas-devel
BuildRequires:  fdupes
BuildRequires:  gcc-fortran
BuildRequires:  lapack-devel
%if %{with mpi}
BuildRequires:  %{mpi_flavor}%{?mpi_ext}-devel
BuildRequires:  fftw3-mpi-devel
BuildRequires:  libscalapack2-%{mpi_flavor}%{?mpi_ext}-devel
%else
BuildRequires:  fftw3-devel
%endif

%description
Quantum ESPRESSO is an integrated suite of Open-Source computer codes for
electronic-structure calculations and materials modeling at the nanoscale.
It is based on density-functional theory, plane waves, and pseudopotentials.

%package doc
Summary:        Documentation files for Quantum Espresso
Group:          Documentation/Other

%description doc
This package contains the documentation files for Quantum Espresso.

Quantum ESPRESSO is an integrated suite of Open-Source computer codes for
electronic-structure calculations and materials modeling at the nanoscale.
It is based on density-functional theory, plane waves, and pseudopotentials.

%prep
%autosetup -p1 -n q-e-qe-%{version}
cp %{SOURCE1} ./external/devxlib/devxlib.tar.gz
# Need to pass -D__FFTW, see https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=980677
# Error: Symbol 'cft_2xy' at (1) has no IMPLICIT type
sed -i 's|MANUAL_DFLAGS  =|MANUAL_DFLAGS  = -D__FFTW %{extra_gfortran_flags}|' install/make.inc.in

%build
# Note: optflags should not be passed to fortran flags as they cause build failures
%if %{with mpi}
export CC="%{my_bindir}/mpicc"
export CFLAGS='%{extra_gfortran_flags} %{optflags}'
export FC="%{my_bindir}/mpif90"
export FCFLAGS='%{extra_gfortran_flags}'
export FFLAGS='%{extra_gfortran_flags}'
export BLAS_LIBS="-L%{_libdir} -lblas"
export LAPACK_LIBS="-L%{_libdir} -llapack"
export FFT_LIBS="-L%{_libdir} -lfftw3_mpi"
export SCALAPACK_LIBS="-L%{my_libdir} -lscalapack"
%configure --enable-parallel
%else
export CC=gcc
export CFLAGS='%{extra_gfortran_flags} %{optflags}'
export FC=gfortran
export FCFLAGS='%{extra_gfortran_flags}'
export FFLAGS='%{extra_gfortran_flags}'
export BLAS_LIBS="-L%{_libdir} -lblas"
export LAPACK_LIBS="-L%{_libdir} -llapack"
export FFT_LIBS="-lfftw3"
%configure --disable-parallel
%endif
%make_build -j1 all

%install
mkdir -p %{buildroot}%{my_bindir}
pushd bin
for bin in *.x; do
    install -m 755 $bin %{buildroot}%{my_bindir}/qe_$bin
done
popd

%fdupes -s Doc/

%files
%doc README.md
%license License
%{my_bindir}/*.x

%if %{without mpi}
%files doc
%doc Doc/*
%endif

%changelog
* Mon Oct 23 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for obsolete openmpi[123]
- Prepare support for openmpi5
* Thu Sep  1 2022 Atri Bhattacharya <badshah400@gmail.com>
- Add quantum-espresso-rpmlintrc file to disable false positives
  about executable stacks in binaries.
- Parallel make leads to race issues causing build failures;
  enforce -j1 when running %%make_build.
* Sun Nov 21 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 6.8:
  * See release notes here:
    <https://gitlab.com/QEF/q-e/-/tags/qe-6.8>.
- Drop backports-6.4.1.git-diff: incorporated upstream.
- Add quantum-espresso-devxlib-no-download.patch: Patch to avoid
  downloading devxlib from the web; we supply it as an additional
  source instead.
- Add devxlib as an additional source and copy it to dir where
  quantum-espresso build expects.
- Add openmpi4 as multibuild flavor.
- Drop mvapich2 multibuild flavor, not well supported and builds
  fail.
- Hack-ish work-arounds to prevent build failures all around
  (locally checked that tests work; but they need further
  downloads and hacks, and is best avoided within the build system
  itself):
  * Turn off errors on return-type warnings in %%optflags.
  * Allow compilation to proceed despite minor rank mis-matches by
    passing -fallow-argument-mismatch as an additional flag to
    fortran compiler (for GCC >= 10 only).
  * Explicitly pass blas, lapack, and fftw3 library flags to
    configure; otherwise the build tried to build each of these
    internally; for fftw3 use parallel or serial library depending
    on mulbuild flavor.
- Link against scalapack for parallel build flavors.
* Mon Feb 10 2020 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Add missing Provides/Obsoletes for renamed openmpi1 package.
- Avoid overriding ExclusiveArch for disabled flavors.
- Cleanup spec file, use same patterns as other MPI packages.
* Wed Jan 29 2020 Atri Bhattacharya <badshah400@gmail.com>
- Remove duplicated definitions.
- Remove bogus undefines of suffix and mpi_flavor for "serial"
  flavour. The former causes builds to fail for openSUSE >= 1550
  using rpm >= 4.15.
* Thu Nov  7 2019 Atri Bhattacharya <badshah400@gmail.com>
- Use multibuild to build different mpi (mvapich2, openmpi1,
  openmpi2, and openmpi3) and serial versions.
* Thu Nov  7 2019 Atri Bhattacharya <badshah400@gmail.com>
- Add backports-6.4.1.git-diff: Backported fixes for version
  6.4.1 from upstream:
  * Bug in spin-polarized meta-GGA
  * Unphysical fractional translations (tau/n with n/=2,3,4,6)
    were not explicitly discarded, thus leading in unfortunate
    cases to strange values for FFT factors and grids. Also: if
    "nosym" is true, inversion symmetry flag (invsym) and info on
    FFT factors (fft_fact) must also be reset
  * PPACF wasn't working any longer in v.6.4 and 6.4.1 for nspin=2
    and for hybrid functionals
  * option "write_unkg" of pw2wannier90.f90 wasn't working as
    expected
  * Input parameters (for restarting DFPT+U calculations)
    read_dns_bare and d2ns_type were missing in the PH input
    namelist, and moreover they were not broadcasted.
* Thu Nov  7 2019 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 6.4.1:
  * A warning is issued if the lattice parameter seems to be a
    conversion factor instead of a true lattice parameter.
  * A warning is issued if ibrav=0 is used for systems having
    symmetry.
  * [HP] the code was not working correctly when fractional
    translations were present
  * [HP] there was a bug in the case when either there is only one
    k point, or when k pools are used and some of the pools have
    only one k point.
  * Restart of ph.x with 2D boundary conditions has been fixed
    (see gitlab issue #102 (closed))
  * XML file correctly written if tetrahedra are used (see gitlab
    issue #103 (closed))
- Major changes since last packaged version including backward
  incomaptible changes: See
  * https://gitlab.com/QEF/q-e/-/tags/qe-6.0.0
  * https://gitlab.com/QEF/q-e/-/tags/qe-6.1.0
  * https://gitlab.com/QEF/q-e/-/tags/qe-6.2.0
  * https://gitlab.com/QEF/q-e/wikis/Releases/Quantum-Espresso-6.3-Release-Notes
  * https://gitlab.com/QEF/q-e/wikis/Releases/Quantum-Espresso-6.4-Release-Notes
- Drop patches no longer compatible or needed with updated
  version:
  * quantum_espresso_do_not_set_xlf_for_powerpc.patch
  * quantum_espresso_add_ppc64le_archi_to_configure.patch
  * espresso-implicit-pointer-decl.patch
* Thu Nov  7 2019 Atri Bhattacharya <badshah400@gmail.com>
- Fix unresolvable status for openSUSE < 1550 where openmpi1 is
  still called openmpi.
* Fri Nov  1 2019 Atri Bhattacharya <badshah400@gmail.com>
- openmpi has been renamed to openmpi1; adapt BuildRequires and
  file list accordingly.
* Thu Aug  6 2015 normand@linux.vnet.ibm.com
- Two new patches to build ppc64/ppc64le archi
  (not yet upstream)
  quantum_espresso_add_ppc64le_archi_to_configure.patch
  quantum_espresso_do_not_set_xlf_for_powerpc.patch
* Sat Mar 14 2015 p.drouand@gmail.com
- Update to version 5.2.1
  * No changelog available
- Use download Url as source
- Remove espresso-5.0.2-no-return-in-nonvoid-function.patch; fixed
  on upstream release
- Update espresso-5.0.2-implicit-pointer-decl.patch >
  espresso-implicit-pointer-decl.patch
- Add xspectra-5.2.1.tar.gz; needed to build package
- TODO file doesn't exist anymore
- Copy README into the package
* Fri May  2 2014 dmitry_r@opensuse.org
- Fix finging of duplicate files in documentation
* Wed Apr 30 2014 dmitry_r@opensuse.org
- Rename package to quantum-espresso to avoid conflict with espresso
* Thu May 23 2013 scorot@free.fr
- first package
- Fix GCC warnings
  * espresso-5.0.2-implicit-pointer-decl.patch
  * espresso-5.0.2-no-return-in-nonvoid-function.patch
